CON

  _CLKMODE = XTAL1 + PLL16X 
  _XINFREQ = 5_000_000
  
  START_LATITUDE = 40.00000
  START_LONGITUDE = -112.0000
  END_LATITUDE = 40.010000
  END_LONGITUDE = -112.0100

  F_KILOMETERS_PER_DEGREE_LAT = 111.1949 ' using mean radius of earth = 6371 km '111.2263 
    
VAR

  long fLongitude[2]
  long fLatitude[2]
           
OBJ

  Pst : "Parallax Serial Terminal"
  FMath : "F32"
    
PUB Setup

  FMath.Start

  Pst.start(115200)

  MainLoop

PUB MainLoop | fDistance
    
  repeat
    fLatitude[0] := START_LATITUDE
    fLongitude[0] := START_LONGITUDE
    fLatitude[1] := END_LATITUDE
    fLongitude[1] := END_LONGITUDE
    fDistance := CalculateDistance(fLatitude[0], fLongitude[0], fLatitude[1], fLongitude[1])
    DisplayDistance(fDistance)
    waitcnt(clkfreq / 10 + cnt)
    
PUB CalculateDistance(fLat1, fLon1, fLat2, fLon2) | fAverageLat

  fAverageLat := FMath.FAdd(fLat1, fLat2)
  fAverageLat := FMath.FDiv(fAverageLat, 2.0)
  
  fLat1 := FMath.FSub(fLat2, fLat1)
  fLon1 := FMath.FSub(fLon2, fLon1)

  fLat1 := FMath.FMul(fLat1, F_KILOMETERS_PER_DEGREE_LAT)
  fLon1 := FMath.FMul(fLon1, KilometersPerDegLongitude(fAverageLat))

  fLat1 := FMath.FMul(fLat1, fLat1)
  fLon1 := FMath.FMul(fLon1, fLon1)

  
  result := FMath.FSqr(FMath.FAdd(fLat1, fLon1))

PUB KilometersPerDegLongitude(localFLatitude)

  result := FMath.FMul(F_KILOMETERS_PER_DEGREE_LAT, FMath.Cos(FMath.Radians(localFLatitude)))

PUB DisplayDistance(fDistance)
         
  Pst.Char(1)                 ' Home    
  Pst.str(string("Distance = "))        
  PrintFloat(fDistance)
  Pst.str(string(" km"))        
  Pst.ClearEnd
  
PUB PrintFloat(floatToPrint)

  result := FMath.FRound(FMath.FMul(floatToPrint, 1000.0))
  DecPoint(result, 1000)
  
PUB DecPoint(value, denominator)
  if value < 0
    Pst.Char("-")
    -value
      
  if value => denominator
    result := value / denominator
    Pst.Dec(result)
    value //= denominator     
  else    
    Pst.Char("0")
  Pst.Char(".")  
  repeat while denominator > 1
    denominator /= 10
    if value => denominator
      result := value / denominator
      Pst.Dec(result)
      value //= denominator
    else
      Pst.Char("0")
  